/*
   written by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   dynamically growing strings on the heap

							06.09.1997
 */

#include <stdarg.h>

#include <X11/Intrinsic.h>

#include "dstring.h"


char  *Xstrdup(char *str)
{
   char *p = XtMalloc((strlen(str) + 1) * sizeof(char));
   strcpy(p, str);
   return(p);
}

char  *Xstrndup(char *str, int n)
{
   char *p = XtMalloc((n + 1) * sizeof(char));
   strncpy(p, str, n);
   p[n] = '\0';
   return(p);
}


#define  DS_STEP  100

void DStringInit(DString *ds)
{
   ds->str = NULL;
   ds->len = 0;
}

void DStringFree(DString *ds)
{
   XtFree(ds->str);
   DStringInit(ds);
}


static void DStringResize(DString *ds, int len)
{
   if (len > ds->len)
   {
      len = ((len + DS_STEP - 1) / DS_STEP) * DS_STEP;
      if (!ds->str)
      {
         ds->str = XtMalloc(len * sizeof(char));
         ds->str[0] = '\0';
      }
      else
         ds->str = XtRealloc(ds->str, len * sizeof(char));
      ds->len = len;
   }
}


void DStringNew(DString *ds, char *str)
{
   DStringInit(ds);
   DStringCat(ds, str);
}

void DStringCompose(DString *ds, ...)
{
   va_list  vl;
   char    *p;

   DStringInit(ds);
   va_start(vl, ds);
   while((p = va_arg(vl, char*)) != NULL)
      DStringCat(ds, p);
   va_end(vl);
}

void DStringCat(DString *ds, char *str)
{
   int  akt_len = ds->str ? strlen(ds->str) : 0;
   int  new_len = akt_len + strlen(str) + 1;
   DStringResize(ds, new_len);
   strcat(ds->str, str);
}

void DStringNCat(DString *ds, char *str, int n)
{
   int  akt_len = ds->str ? strlen(ds->str) : 0;
   int  new_len = akt_len + n + 1;
   DStringResize(ds, new_len);
   strncat(ds->str, str, n);
}
